@@ -14,11 +14,8 @@
 //!
 //! This API is completely unstable and subject to change.
 
-// Do not remove on snapshot creation. Needed for bootstrap. (Issue #22364)
-#![cfg_attr(stage0, feature(custom_attribute))]
 #![crate_name = "rustc_driver"]
 #![unstable(feature = "rustc_private", issue = "27812")]
-#![staged_api]
 #![crate_type = "dylib"]
 #![crate_type = "rlib"]
 #![doc(html_logo_url = "https://www.rust-lang.org/logos/rust-logo-128x128-blk-v2.png",
@@ -32,7 +29,6 @@
 #![feature(rustc_private)]
 #![feature(set_stdio)]
 #![feature(staged_api)]
-#![feature(vec_push_all)]
 
 extern crate arena;
 extern crate flate;
@@ -42,19 +38,23 @@ extern crate libc;
 extern crate rustc;
 extern crate rustc_back;
 extern crate rustc_borrowck;
+extern crate rustc_passes;
 extern crate rustc_front;
 extern crate rustc_lint;
+extern crate rustc_plugin;
 extern crate rustc_privacy;
+extern crate rustc_metadata;
 extern crate rustc_mir;
 extern crate rustc_resolve;
 extern crate rustc_trans;
 extern crate rustc_typeck;
 extern crate serialize;
 extern crate rustc_llvm as llvm;
-#[macro_use] extern crate log;
-#[macro_use] extern crate syntax;
-
-pub use syntax::diagnostic;
+#[macro_use]
+extern crate log;
+#[macro_use]
+extern crate syntax;
+extern crate syntax_ext;
 
 use driver::CompileController;
 use pretty::{PpMode, UserIdentifiedItem};
@@ -63,18 +63,23 @@ use rustc_resolve as resolve;
 use rustc_trans::back::link;
 use rustc_trans::save;
 use rustc::session::{config, Session, build_session};
-use rustc::session::config::{Input, PrintRequest, OutputType};
+use rustc::session::config::{Input, PrintRequest, OutputType, ErrorOutputType};
+use rustc::middle::cstore::CrateStore;
 use rustc::lint::Lint;
 use rustc::lint;
-use rustc::metadata;
+use rustc_metadata::loader;
+use rustc_metadata::cstore::CStore;
 use rustc::util::common::time;
 
+use std::cmp::max;
 use std::cmp::Ordering::Equal;
+use std::default::Default;
 use std::env;
 use std::io::{self, Read, Write};
 use std::iter::repeat;
 use std::path::PathBuf;
 use std::process;
+use std::rc::Rc;
 use std::str;
 use std::sync::{Arc, Mutex};
 use std::thread;
@@ -83,8 +88,10 @@ use rustc::session::early_error;
 
 use syntax::ast;
 use syntax::parse;
-use syntax::diagnostic::Emitter;
+use syntax::errors;
+use syntax::errors::emitter::Emitter;
 use syntax::diagnostics;
+use syntax::parse::token;
 
 #[cfg(test)]
 pub mod test;
@@ -94,9 +101,8 @@ pub mod pretty;
 pub mod target_features;
 
 
-const BUG_REPORT_URL: &'static str =
-    "https://github.com/rust-lang/rust/blob/master/CONTRIBUTING.md#bug-reports";
-
+const BUG_REPORT_URL: &'static str = "https://github.com/rust-lang/rust/blob/master/CONTRIBUTING.\
+                                      md#bug-reports";
 
 pub fn run(args: Vec<String>) -> isize {
     monitor(move || run_compiler(&args, &mut RustcDefaultCalls));
@@ -105,8 +111,7 @@ pub fn run(args: Vec<String>) -> isize {
 
 // Parse args and run the compiler. This is the primary entry point for rustc.
 // See comments on CompilerCalls below for details about the callbacks argument.
-pub fn run_compiler<'a>(args: &[String],
-                        callbacks: &mut CompilerCalls<'a>) {
+pub fn run_compiler<'a>(args: &[String], callbacks: &mut CompilerCalls<'a>) {
     macro_rules! do_or_return {($expr: expr) => {
         match $expr {
             Compilation::Stop => return,
@@ -116,29 +121,28 @@ pub fn run_compiler<'a>(args: &[String],
 
     let matches = match handle_options(args.to_vec()) {
         Some(matches) => matches,
-        None => return
+        None => return,
     };
 
     let sopts = config::build_session_options(&matches);
 
     let descriptions = diagnostics_registry();
 
-    do_or_return!(callbacks.early_callback(&matches, &descriptions, sopts.color));
+    do_or_return!(callbacks.early_callback(&matches, &descriptions, sopts.error_format));
 
     let (odir, ofile) = make_output(&matches);
     let (input, input_file_path) = match make_input(&matches.free) {
         Some((input, input_file_path)) => callbacks.some_input(input, input_file_path),
         None => match callbacks.no_input(&matches, &sopts, &odir, &ofile, &descriptions) {
             Some((input, input_file_path)) => (input, input_file_path),
-            None => return
-        }
+            None => return,
+        },
     };
 
-    let mut sess = build_session(sopts, input_file_path, descriptions);
+    let cstore = Rc::new(CStore::new(token::get_ident_interner()));
+    let sess = build_session(sopts, input_file_path, descriptions,
+                                 cstore.clone());
     rustc_lint::register_builtins(&mut sess.lint_store.borrow_mut(), Some(&sess));
-    if sess.unstable_options() {
-        sess.opts.show_span = matches.opt_str("show-span");
-    }
     let mut cfg = config::build_configuration(&sess);
     target_features::add_configuration(&mut cfg, &sess);
 
@@ -149,15 +153,18 @@ pub fn run_compiler<'a>(args: &[String],
     let pretty = callbacks.parse_pretty(&sess, &matches);
     match pretty {
         Some((ppm, opt_uii)) => {
-            pretty::pretty_print_input(sess, cfg, &input, ppm, opt_uii, ofile);
+            pretty::pretty_print_input(sess, &cstore, cfg, &input, ppm, opt_uii, ofile);
             return;
         }
-        None => {/* continue */ }
+        None => {
+            // continue
+        }
     }
 
     let plugins = sess.opts.debugging_opts.extra_plugins.clone();
     let control = callbacks.build_controller(&sess);
-    driver::compile_input(sess, cfg, &input, &odir, &ofile, Some(plugins), control);
+    driver::compile_input(sess, &cstore, cfg, &input, &odir, &ofile,
+                          Some(plugins), control);
 }
 
 // Extract output directory and file from matches.
@@ -176,7 +183,8 @@ fn make_input(free_matches: &[String]) -> Option<(Input, Option<PathBuf>)> {
             io::stdin().read_to_string(&mut src).unwrap();
             Some((Input::Str(src), None))
         } else {
-            Some((Input::File(PathBuf::from(ifile)), Some(PathBuf::from(ifile))))
+            Some((Input::File(PathBuf::from(ifile)),
+                  Some(PathBuf::from(ifile))))
         }
     } else {
         None
@@ -194,7 +202,7 @@ impl Compilation {
     pub fn and_then<F: FnOnce() -> Compilation>(self, next: F) -> Compilation {
         match self {
             Compilation::Stop => Compilation::Stop,
-            Compilation::Continue => next()
+            Compilation::Continue => next(),
         }
     }
 }
@@ -208,7 +216,7 @@ pub trait CompilerCalls<'a> {
     fn early_callback(&mut self,
                       _: &getopts::Matches,
                       _: &diagnostics::registry::Registry,
-                      _: diagnostic::ColorConfig)
+                      _: ErrorOutputType)
                       -> Compilation {
         Compilation::Continue
     }
@@ -229,7 +237,9 @@ pub trait CompilerCalls<'a> {
     // Called after we extract the input from the arguments. Gives the implementer
     // an opportunity to change the inputs or to add some custom input handling.
     // The default behaviour is to simply pass through the inputs.
-    fn some_input(&mut self, input: Input, input_path: Option<PathBuf>)
+    fn some_input(&mut self,
+                  input: Input,
+                  input_path: Option<PathBuf>)
                   -> (Input, Option<PathBuf>) {
         (input, input_path)
     }
@@ -269,7 +279,8 @@ pub trait CompilerCalls<'a> {
         None
     }
 
-    // Create a CompilController struct for controlling the behaviour of compilation.
+    // Create a CompilController struct for controlling the behaviour of
+    // compilation.
     fn build_controller(&mut self, &Session) -> CompileController<'a>;
 }
 
@@ -281,7 +292,7 @@ impl<'a> CompilerCalls<'a> for RustcDefaultCalls {
     fn early_callback(&mut self,
                       matches: &getopts::Matches,
                       descriptions: &diagnostics::registry::Registry,
-                      color: diagnostic::ColorConfig)
+                      output: ErrorOutputType)
                       -> Compilation {
         match matches.opt_str("explain") {
             Some(ref code) => {
@@ -296,12 +307,12 @@ impl<'a> CompilerCalls<'a> for RustcDefaultCalls {
                         print!("{}", &description[1..]);
                     }
                     None => {
-                        early_error(color, &format!("no extended information for {}", code));
+                        early_error(output, &format!("no extended information for {}", code));
                     }
                 }
                 return Compilation::Stop;
-            },
-            None => ()
+            }
+            None => (),
         }
 
         return Compilation::Continue;
@@ -322,16 +333,18 @@ impl<'a> CompilerCalls<'a> for RustcDefaultCalls {
                     describe_lints(&ls, false);
                     return None;
                 }
-                let sess = build_session(sopts.clone(), None, descriptions.clone());
+                let cstore = Rc::new(CStore::new(token::get_ident_interner()));
+                let sess = build_session(sopts.clone(), None, descriptions.clone(),
+                                         cstore.clone());
                 rustc_lint::register_builtins(&mut sess.lint_store.borrow_mut(), Some(&sess));
                 let should_stop = RustcDefaultCalls::print_crate_info(&sess, None, odir, ofile);
                 if should_stop == Compilation::Stop {
                     return None;
                 }
-                early_error(sopts.color, "no input filename given");
+                early_error(sopts.error_format, "no input filename given");
             }
             1 => panic!("make_input should have provided valid inputs"),
-            _ => early_error(sopts.color, "multiple input filenames provided")
+            _ => early_error(sopts.error_format, "multiple input filenames provided"),
         }
 
         None
@@ -366,15 +379,14 @@ impl<'a> CompilerCalls<'a> for RustcDefaultCalls {
                      odir: &Option<PathBuf>,
                      ofile: &Option<PathBuf>)
                      -> Compilation {
-        RustcDefaultCalls::print_crate_info(sess, Some(input), odir, ofile).and_then(
-            || RustcDefaultCalls::list_metadata(sess, matches, input))
+        RustcDefaultCalls::print_crate_info(sess, Some(input), odir, ofile)
+            .and_then(|| RustcDefaultCalls::list_metadata(sess, matches, input))
     }
 
     fn build_controller(&mut self, sess: &Session) -> CompileController<'a> {
         let mut control = CompileController::basic();
 
-        if sess.opts.parse_only ||
-           sess.opts.show_span.is_some() ||
+        if sess.opts.parse_only || sess.opts.debugging_opts.show_span.is_some() ||
            sess.opts.debugging_opts.ast_json_noexpand {
             control.after_parse.stop = Compilation::Stop;
         }
@@ -393,14 +405,14 @@ impl<'a> CompilerCalls<'a> for RustcDefaultCalls {
 
         if sess.opts.debugging_opts.save_analysis {
             control.after_analysis.callback = box |state| {
-                time(state.session.time_passes(),
-                     "save analysis",
-                     || save::process_crate(state.tcx.unwrap(),
-                                            state.lcx.unwrap(),
-                                            state.krate.unwrap(),
-                                            state.analysis.unwrap(),
-                                            state.crate_name.unwrap(),
-                                            state.out_dir));
+                time(state.session.time_passes(), "save analysis", || {
+                    save::process_crate(state.tcx.unwrap(),
+                                        state.lcx.unwrap(),
+                                        state.krate.unwrap(),
+                                        state.analysis.unwrap(),
+                                        state.crate_name.unwrap(),
+                                        state.out_dir)
+                });
             };
             control.make_glob_map = resolve::MakeGlobMap::Yes;
         }
@@ -410,23 +422,19 @@ impl<'a> CompilerCalls<'a> for RustcDefaultCalls {
 }
 
 impl RustcDefaultCalls {
-    pub fn list_metadata(sess: &Session,
-                         matches: &getopts::Matches,
-                         input: &Input)
-                         -> Compilation {
+    pub fn list_metadata(sess: &Session, matches: &getopts::Matches, input: &Input) -> Compilation {
         let r = matches.opt_strs("Z");
         if r.contains(&("ls".to_string())) {
             match input {
                 &Input::File(ref ifile) => {
                     let path = &(*ifile);
                     let mut v = Vec::new();
-                    metadata::loader::list_file_metadata(&sess.target.target,
-                                                         path,
-                                                         &mut v).unwrap();
+                    loader::list_file_metadata(&sess.target.target, path, &mut v)
+                        .unwrap();
                     println!("{}", String::from_utf8(v).unwrap());
                 }
                 &Input::Str(_) => {
-                    early_error(sess.opts.color, "cannot list metadata for stdin");
+                    early_error(ErrorOutputType::default(), "cannot list metadata for stdin");
                 }
             }
             return Compilation::Stop;
@@ -453,29 +461,24 @@ impl RustcDefaultCalls {
                 PrintRequest::CrateName => {
                     let input = match input {
                         Some(input) => input,
-                        None => early_error(sess.opts.color, "no input file provided"),
+                        None => early_error(ErrorOutputType::default(), "no input file provided"),
                     };
                     let attrs = attrs.as_ref().unwrap();
-                    let t_outputs = driver::build_output_filenames(input,
-                                                                   odir,
-                                                                   ofile,
-                                                                   attrs,
-                                                                   sess);
-                    let id = link::find_crate_name(Some(sess),
-                                                   attrs,
-                                                   input);
+                    let t_outputs = driver::build_output_filenames(input, odir, ofile, attrs, sess);
+                    let id = link::find_crate_name(Some(sess), attrs, input);
                     if *req == PrintRequest::CrateName {
                         println!("{}", id);
-                        continue
+                        continue;
                     }
                     let crate_types = driver::collect_crate_types(sess, attrs);
                     let metadata = driver::collect_crate_metadata(sess, attrs);
                     *sess.crate_metadata.borrow_mut() = metadata;
                     for &style in &crate_types {
-                        let fname = link::filename_for_input(sess, style, &id,
-                                                             &t_outputs);
-                        println!("{}", fname.file_name().unwrap()
-                                            .to_string_lossy());
+                        let fname = link::filename_for_input(sess, style, &id, &t_outputs);
+                        println!("{}",
+                                 fname.file_name()
+                                      .unwrap()
+                                      .to_string_lossy());
                     }
                 }
             }
@@ -503,9 +506,13 @@ pub fn commit_date_str() -> Option<&'static str> {
 pub fn version(binary: &str, matches: &getopts::Matches) {
     let verbose = matches.opt_present("verbose");
 
-    println!("{} {}", binary, option_env!("CFG_VERSION").unwrap_or("unknown version"));
+    println!("{} {}",
+             binary,
+             option_env!("CFG_VERSION").unwrap_or("unknown version"));
     if verbose {
-        fn unw(x: Option<&str>) -> &str { x.unwrap_or("unknown") }
+        fn unw(x: Option<&str>) -> &str {
+            x.unwrap_or("unknown")
+        }
         println!("binary: {}", binary);
         println!("commit-hash: {}", unw(commit_hash_str()));
         println!("commit-date: {}", unw(commit_date_str()));
@@ -520,32 +527,35 @@ fn usage(verbose: bool, include_unstable_options: bool) {
     } else {
         config::rustc_short_optgroups()
     };
-    let groups : Vec<_> = groups.into_iter()
-        .filter(|x| include_unstable_options || x.is_stable())
-        .map(|x|x.opt_group)
-        .collect();
+    let groups: Vec<_> = groups.into_iter()
+                               .filter(|x| include_unstable_options || x.is_stable())
+                               .map(|x| x.opt_group)
+                               .collect();
     let message = format!("Usage: rustc [OPTIONS] INPUT");
     let extra_help = if verbose {
         ""
     } else {
         "\n    --help -v           Print the full set of options rustc accepts"
     };
-    println!("{}\n\
-Additional help:
+    println!("{}\nAdditional help:
     -C help             Print codegen options
-    -W help             Print 'lint' options and default settings
-    -Z help             Print internal options for debugging rustc{}\n",
-              getopts::usage(&message, &groups),
-              extra_help);
+    -W help             \
+              Print 'lint' options and default settings
+    -Z help             Print internal \
+              options for debugging rustc{}\n",
+             getopts::usage(&message, &groups),
+             extra_help);
 }
 
 fn describe_lints(lint_store: &lint::LintStore, loaded_plugins: bool) {
     println!("
 Available lint options:
     -W <foo>           Warn about <foo>
-    -A <foo>           Allow <foo>
+    -A <foo>           \
+              Allow <foo>
     -D <foo>           Deny <foo>
-    -F <foo>           Forbid <foo> (deny, and deny all overrides)
+    -F <foo>           Forbid <foo> \
+              (deny, and deny all overrides)
 
 ");
 
@@ -562,7 +572,7 @@ Available lint options:
     }
 
     fn sort_lint_groups(lints: Vec<(&'static str, Vec<lint::LintId>, bool)>)
-                     -> Vec<(&'static str, Vec<lint::LintId>)> {
+                        -> Vec<(&'static str, Vec<lint::LintId>)> {
         let mut lints: Vec<_> = lints.into_iter().map(|(x, y, _)| (x, y)).collect();
         lints.sort_by(|&(x, _): &(&'static str, Vec<lint::LintId>),
                        &(y, _): &(&'static str, Vec<lint::LintId>)| {
@@ -572,21 +582,28 @@ Available lint options:
     }
 
     let (plugin, builtin): (Vec<_>, _) = lint_store.get_lints()
-        .iter().cloned().partition(|&(_, p)| p);
+                                                   .iter()
+                                                   .cloned()
+                                                   .partition(|&(_, p)| p);
     let plugin = sort_lints(plugin);
     let builtin = sort_lints(builtin);
 
     let (plugin_groups, builtin_groups): (Vec<_>, _) = lint_store.get_lint_groups()
-        .iter().cloned().partition(|&(_, _, p)| p);
+                                                                 .iter()
+                                                                 .cloned()
+                                                                 .partition(|&(_, _, p)| p);
     let plugin_groups = sort_lint_groups(plugin_groups);
     let builtin_groups = sort_lint_groups(builtin_groups);
 
-    let max_name_len = plugin.iter().chain(&builtin)
-        .map(|&s| s.name.chars().count())
-        .max().unwrap_or(0);
+    let max_name_len = plugin.iter()
+                             .chain(&builtin)
+                             .map(|&s| s.name.chars().count())
+                             .max()
+                             .unwrap_or(0);
     let padded = |x: &str| {
-        let mut s = repeat(" ").take(max_name_len - x.chars().count())
-                               .collect::<String>();
+        let mut s = repeat(" ")
+                        .take(max_name_len - x.chars().count())
+                        .collect::<String>();
         s.push_str(x);
         s
     };
@@ -599,7 +616,9 @@ Available lint options:
         for lint in lints {
             let name = lint.name_lower().replace("_", "-");
             println!("    {}  {:7.7}  {}",
-                     padded(&name[..]), lint.default_level.as_str(), lint.desc);
+                     padded(&name[..]),
+                     lint.default_level.as_str(),
+                     lint.desc);
         }
         println!("\n");
     };
@@ -608,12 +627,17 @@ Available lint options:
 
 
 
-    let max_name_len = plugin_groups.iter().chain(&builtin_groups)
-        .map(|&(s, _)| s.chars().count())
-        .max().unwrap_or(0);
+    let max_name_len = max("warnings".len(),
+                           plugin_groups.iter()
+                                        .chain(&builtin_groups)
+                                        .map(|&(s, _)| s.chars().count())
+                                        .max()
+                                        .unwrap_or(0));
+
     let padded = |x: &str| {
-        let mut s = repeat(" ").take(max_name_len - x.chars().count())
-                               .collect::<String>();
+        let mut s = repeat(" ")
+                        .take(max_name_len - x.chars().count())
+                        .collect::<String>();
         s.push_str(x);
         s
     };
@@ -621,14 +645,16 @@ Available lint options:
     println!("Lint groups provided by rustc:\n");
     println!("    {}  {}", padded("name"), "sub-lints");
     println!("    {}  {}", padded("----"), "---------");
+    println!("    {}  {}", padded("warnings"), "all built-in lints");
 
     let print_lint_groups = |lints: Vec<(&'static str, Vec<lint::LintId>)>| {
         for (name, to) in lints {
             let name = name.to_lowercase().replace("_", "-");
-            let desc = to.into_iter().map(|x| x.as_str().replace("_", "-"))
-                         .collect::<Vec<String>>().join(", ");
-            println!("    {}  {}",
-                     padded(&name[..]), desc);
+            let desc = to.into_iter()
+                         .map(|x| x.as_str().replace("_", "-"))
+                         .collect::<Vec<String>>()
+                         .join(", ");
+            println!("    {}  {}", padded(&name[..]), desc);
         }
         println!("\n");
     };
@@ -657,25 +683,38 @@ Available lint options:
 
 fn describe_debug_flags() {
     println!("\nAvailable debug options:\n");
-    for &(name, _, opt_type_desc, desc) in config::DB_OPTIONS {
-        let (width, extra) = match opt_type_desc {
-            Some(..) => (21, "=val"),
-            None => (25, "")
-        };
-        println!("    -Z {:>width$}{} -- {}", name.replace("_", "-"),
-                 extra, desc, width=width);
-    }
+    print_flag_list("-Z", config::DB_OPTIONS);
 }
 
 fn describe_codegen_flags() {
     println!("\nAvailable codegen options:\n");
-    for &(name, _, opt_type_desc, desc) in config::CG_OPTIONS {
+    print_flag_list("-C", config::CG_OPTIONS);
+}
+
+fn print_flag_list<T>(cmdline_opt: &str,
+                      flag_list: &[(&'static str, T, Option<&'static str>, &'static str)]) {
+    let max_len = flag_list.iter()
+                           .map(|&(name, _, opt_type_desc, _)| {
+                               let extra_len = match opt_type_desc {
+                                   Some(..) => 4,
+                                   None => 0,
+                               };
+                               name.chars().count() + extra_len
+                           })
+                           .max()
+                           .unwrap_or(0);
+
+    for &(name, _, opt_type_desc, desc) in flag_list {
         let (width, extra) = match opt_type_desc {
-            Some(..) => (21, "=val"),
-            None => (25, "")
+            Some(..) => (max_len - 4, "=val"),
+            None => (max_len, ""),
         };
-        println!("    -C {:>width$}{} -- {}", name.replace("_", "-"),
-                 extra, desc, width=width);
+        println!("    {} {:>width$}{} -- {}",
+                 cmdline_opt,
+                 name.replace("_", "-"),
+                 extra,
+                 desc,
+                 width = width);
     }
 }
 
@@ -699,8 +738,10 @@ pub fn handle_options(mut args: Vec<String>) -> Option<getopts::Matches> {
     }
 
     fn parse_all_options(args: &Vec<String>) -> getopts::Matches {
-        let all_groups : Vec<getopts::OptGroup>
-            = config::rustc_optgroups().into_iter().map(|x|x.opt_group).collect();
+        let all_groups: Vec<getopts::OptGroup> = config::rustc_optgroups()
+                                                     .into_iter()
+                                                     .map(|x| x.opt_group)
+                                                     .collect();
         match getopts::getopts(&args[..], &all_groups) {
             Ok(m) => {
                 if !allows_unstable_options(&m) {
@@ -713,15 +754,16 @@ pub fn handle_options(mut args: Vec<String>) -> Option<getopts::Matches> {
                             &opt.opt_group.short_name
                         };
                         if m.opt_present(opt_name) {
-                            early_error(diagnostic::Auto, &format!("use of unstable option '{}' \
-                                                                    requires -Z unstable-options",
-                                                                   opt_name));
+                            early_error(ErrorOutputType::default(),
+                                        &format!("use of unstable option '{}' requires -Z \
+                                                  unstable-options",
+                                                 opt_name));
                         }
                     }
                 }
                 m
             }
-            Err(f) => early_error(diagnostic::Auto, &f.to_string())
+            Err(f) => early_error(ErrorOutputType::default(), &f.to_string()),
         }
     }
 
@@ -744,7 +786,8 @@ pub fn handle_options(mut args: Vec<String>) -> Option<getopts::Matches> {
     };
 
     if matches.opt_present("h") || matches.opt_present("help") {
-        usage(matches.opt_present("verbose"), allows_unstable_options(&matches));
+        usage(matches.opt_present("verbose"),
+              allows_unstable_options(&matches));
         return None;
     }
 
@@ -763,7 +806,9 @@ pub fn handle_options(mut args: Vec<String>) -> Option<getopts::Matches> {
     }
 
     if cg_flags.contains(&"passes=list".to_string()) {
-        unsafe { ::llvm::LLVMRustPrintPasses(); }
+        unsafe {
+            ::llvm::LLVMRustPrintPasses();
+        }
         return None;
     }
 
@@ -775,20 +820,16 @@ pub fn handle_options(mut args: Vec<String>) -> Option<getopts::Matches> {
     Some(matches)
 }
 
-fn parse_crate_attrs(sess: &Session, input: &Input) ->
-                     Vec<ast::Attribute> {
+fn parse_crate_attrs(sess: &Session, input: &Input) -> Vec<ast::Attribute> {
     let result = match *input {
         Input::File(ref ifile) => {
-            parse::parse_crate_attrs_from_file(ifile,
-                                               Vec::new(),
-                                               &sess.parse_sess)
+            parse::parse_crate_attrs_from_file(ifile, Vec::new(), &sess.parse_sess)
         }
         Input::Str(ref src) => {
-            parse::parse_crate_attrs_from_source_str(
-                driver::anon_src().to_string(),
-                src.to_string(),
-                Vec::new(),
-                &sess.parse_sess)
+            parse::parse_crate_attrs_from_source_str(driver::anon_src().to_string(),
+                                                     src.to_string(),
+                                                     Vec::new(),
+                                                     &sess.parse_sess)
         }
     };
     result.into_iter().collect()
@@ -799,7 +840,7 @@ fn parse_crate_attrs(sess: &Session, input: &Input) ->
 ///
 /// The diagnostic emitter yielded to the procedure should be used for reporting
 /// errors of the compiler.
-pub fn monitor<F:FnOnce()+Send+'static>(f: F) {
+pub fn monitor<F: FnOnce() + Send + 'static>(f: F) {
     const STACK_SIZE: usize = 8 * 1024 * 1024; // 8MB
 
     struct Sink(Arc<Mutex<Vec<u8>>>);
@@ -807,7 +848,9 @@ pub fn monitor<F:FnOnce()+Send+'static>(f: F) {
         fn write(&mut self, data: &[u8]) -> io::Result<usize> {
             Write::write(&mut *self.0.lock().unwrap(), data)
         }
-        fn flush(&mut self) -> io::Result<()> { Ok(()) }
+        fn flush(&mut self) -> io::Result<()> {
+            Ok(())
+        }
     }
 
     let data = Arc::new(Mutex::new(Vec::new()));
@@ -821,34 +864,36 @@ pub fn monitor<F:FnOnce()+Send+'static>(f: F) {
         cfg = cfg.stack_size(STACK_SIZE);
     }
 
-    match cfg.spawn(move || { io::set_panic(box err); f() }).unwrap().join() {
-        Ok(()) => { /* fallthrough */ }
+    match cfg.spawn(move || {
+                 io::set_panic(box err);
+                 f()
+             })
+             .unwrap()
+             .join() {
+        Ok(()) => {
+            // fallthrough
+        }
         Err(value) => {
             // Thread panicked without emitting a fatal diagnostic
-            if !value.is::<diagnostic::FatalError>() {
-                let mut emitter = diagnostic::EmitterWriter::stderr(diagnostic::Auto, None);
+            if !value.is::<errors::FatalError>() {
+                let mut emitter = errors::emitter::BasicEmitter::stderr(errors::ColorConfig::Auto);
 
                 // a .span_bug or .bug call has already printed what
                 // it wants to print.
-                if !value.is::<diagnostic::ExplicitBug>() {
-                    emitter.emit(
-                        None,
-                        "unexpected panic",
-                        None,
-                        diagnostic::Bug);
+                if !value.is::<errors::ExplicitBug>() {
+                    emitter.emit(None, "unexpected panic", None, errors::Level::Bug);
                 }
 
-                let xs = [
-                    "the compiler unexpectedly panicked. this is a bug.".to_string(),
-                    format!("we would appreciate a bug report: {}",
-                            BUG_REPORT_URL),
-                ];
+                let xs = ["the compiler unexpectedly panicked. this is a bug.".to_string(),
+                          format!("we would appreciate a bug report: {}", BUG_REPORT_URL)];
                 for note in &xs {
-                    emitter.emit(None, &note[..], None, diagnostic::Note)
+                    emitter.emit(None, &note[..], None, errors::Level::Note)
                 }
                 if let None = env::var_os("RUST_BACKTRACE") {
-                    emitter.emit(None, "run with `RUST_BACKTRACE=1` for a backtrace",
-                                 None, diagnostic::Note);
+                    emitter.emit(None,
+                                 "run with `RUST_BACKTRACE=1` for a backtrace",
+                                 None,
+                                 errors::Level::Note);
                 }
 
                 println!("{}", str::from_utf8(&data.lock().unwrap()).unwrap());
@@ -867,10 +912,12 @@ pub fn diagnostics_registry() -> diagnostics::registry::Registry {
     use syntax::diagnostics::registry::Registry;
 
     let mut all_errors = Vec::new();
-    all_errors.push_all(&rustc::DIAGNOSTICS);
-    all_errors.push_all(&rustc_typeck::DIAGNOSTICS);
-    all_errors.push_all(&rustc_borrowck::DIAGNOSTICS);
-    all_errors.push_all(&rustc_resolve::DIAGNOSTICS);
+    all_errors.extend_from_slice(&rustc::DIAGNOSTICS);
+    all_errors.extend_from_slice(&rustc_typeck::DIAGNOSTICS);
+    all_errors.extend_from_slice(&rustc_borrowck::DIAGNOSTICS);
+    all_errors.extend_from_slice(&rustc_resolve::DIAGNOSTICS);
+    all_errors.extend_from_slice(&rustc_privacy::DIAGNOSTICS);
+    all_errors.extend_from_slice(&rustc_trans::DIAGNOSTICS);
 
     Registry::new(&*all_errors)
 }
