@@ -17,9 +17,10 @@ Rust MIR: a lowered representation of Rust. Also: an experiment!
 #![crate_name = "rustc_mir"]
 #![crate_type = "rlib"]
 #![crate_type = "dylib"]
+#![unstable(feature = "rustc_private", issue = "27812")]
 
 #![feature(rustc_private)]
-#![feature(into_cow)]
+#![feature(staged_api)]
 
 #[macro_use] extern crate log;
 extern crate graphviz as dot;
@@ -30,8 +31,8 @@ extern crate rustc_back;
 extern crate syntax;
 
 pub mod build;
+pub mod graphviz;
+mod hair;
 pub mod mir_map;
-pub mod hair;
-pub mod repr;
-mod graphviz;
-mod tcx;
+pub mod pretty;
+pub mod transform;
