@@ -12,6 +12,7 @@
 //
 // Code relating to drop glue.
 
+use std;
 
 use back::link::*;
 use llvm;
@@ -87,6 +88,10 @@ pub fn trans_exchange_free_ty<'blk, 'tcx>(bcx: Block<'blk, 'tcx>,
     }
 }
 
+fn type_needs_drop<'tcx>(tcx: &ty::ctxt<'tcx>, ty: Ty<'tcx>) -> bool {
+    tcx.type_needs_drop_given_env(ty, &tcx.empty_parameter_environment())
+}
+
 pub fn get_drop_glue_type<'a, 'tcx>(ccx: &CrateContext<'a, 'tcx>,
                                     t: Ty<'tcx>) -> Ty<'tcx> {
     let tcx = ccx.tcx();
@@ -105,11 +110,11 @@ pub fn get_drop_glue_type<'a, 'tcx>(ccx: &CrateContext<'a, 'tcx>,
     // returned `tcx.types.i8` does not appear unsound. The impact on
     // code quality is unknown at this time.)
 
-    if !type_needs_drop(tcx, t) {
+    if !type_needs_drop(&tcx, t) {
         return tcx.types.i8;
     }
     match t.sty {
-        ty::TyBox(typ) if !type_needs_drop(tcx, typ)
+        ty::TyBox(typ) if !type_needs_drop(&tcx, typ)
                          && type_is_sized(tcx, typ) => {
             let llty = sizing_type_of(ccx, typ);
             // `Box<ZeroSizeType>` does not allocate.
@@ -433,14 +438,21 @@ pub fn size_and_align_of_dst<'blk, 'tcx>(bcx: Block<'blk, 'tcx>, t: Ty<'tcx>, in
 
             // Choose max of two known alignments (combined value must
             // be aligned according to more restrictive of the two).
-            let align = Select(bcx,
-                               ICmp(bcx,
-                                    llvm::IntUGT,
-                                    sized_align,
-                                    unsized_align,
-                                    dbloc),
-                               sized_align,
-                               unsized_align);
+            let align = match (const_to_opt_uint(sized_align), const_to_opt_uint(unsized_align)) {
+                (Some(sized_align), Some(unsized_align)) => {
+                    // If both alignments are constant, (the sized_align should always be), then
+                    // pick the correct alignment statically.
+                    C_uint(ccx, std::cmp::max(sized_align, unsized_align))
+                }
+                _ => Select(bcx,
+                            ICmp(bcx,
+                                 llvm::IntUGT,
+                                 sized_align,
+                                 unsized_align,
+                                 dbloc),
+                            sized_align,
+                            unsized_align)
+            };
 
             // Issue #27023: must add any necessary padding to `size`
             // (to make it a multiple of `align`) before returning it.
@@ -451,7 +463,7 @@ pub fn size_and_align_of_dst<'blk, 'tcx>(bcx: Block<'blk, 'tcx>, t: Ty<'tcx>, in
             //
             // emulated via the semi-standard fast bit trick:
             //
-            //   `(size + (align-1)) & !align`
+            //   `(size + (align-1)) & -align`
 
             let addend = Sub(bcx, align, C_uint(bcx.ccx(), 1_u64), dbloc);
             let size = And(
