@@ -15,10 +15,11 @@
 
 #[macro_use] extern crate rustc;
 extern crate rustc_front;
+extern crate rustc_plugin;
 extern crate syntax;
 
 use rustc::lint::{LateContext, LintContext, LintPass, LateLintPass, LateLintPassObject, LintArray};
-use rustc::plugin::Registry;
+use rustc_plugin::Registry;
 use rustc_front::hir;
 use syntax::attr;
 
