@@ -17,5 +17,6 @@ extern crate const_fn_lib;
 use const_fn_lib::foo;
 
 fn main() {
-    let x: [usize; foo()] = []; //~ ERROR non-constant path in constant expr
+    let x: [usize; foo()] = [];
+    //~^ ERROR unimplemented constant expression: calling non-local const fn [E0250]
 }
