@@ -1,4 +1,4 @@
-// Copyright 2013-2014 The Rust Project Developers. See the COPYRIGHT
+// Copyright 2013-2015 The Rust Project Developers. See the COPYRIGHT
 // file at the top-level directory of this distribution and at
 // http://rust-lang.org/COPYRIGHT.
 //
@@ -8,16 +8,8 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-// aux-build:issue_3907.rs
-extern crate issue_3907;
-
-type Foo = issue_3907::Foo+'static;
-
-struct S {
-    name: isize
+fn main() {
+    // Unconstrained type:
+    format!("{:?}", None);
+    //~^ ERROR type annotations or generic parameter binding required
 }
-
-fn bar(_x: Foo) {}
-//~^ ERROR E0277
-
-fn main() {}
