@@ -14,8 +14,7 @@
 trait TheTrait<'t>: 't { }
 
 struct Foo<'a,'b> {
-    x: Box<TheTrait<'a>+'b>
-        //~^ ERROR reference has a longer lifetime
+    x: Box<TheTrait<'a>+'b> //~ ERROR E0478
 }
 
 fn main() { }
