@@ -16,10 +16,10 @@ impl bar for u32 { fn dup(&self) -> u32 { *self } fn blah<X>(&self) {} }
 
 fn main() {
     10.dup::<i32>(); //~ ERROR does not take type parameters
-    10.blah::<i32, i32>(); //~ ERROR incorrect number of type parameters
+    10.blah::<i32, i32>();
+    //~^ ERROR incorrect number of type parameters given for this method: expected 1, found 2
     (box 10 as Box<bar>).dup();
     //~^ ERROR E0038
     //~| ERROR E0038
     //~| ERROR E0277
-    //~| WARNING E0038
 }
