@@ -18,10 +18,10 @@ struct Bar<T:Eq+?Sized> { value: Box<T> }
 
 trait Foo {
     fn bar(&self) where Bar<Self>: Copy;
-        //~^ WARN E0277
+        //~^ ERROR E0277
         //
         // Here, Eq ought to be implemented.
 }
 
 #[rustc_error]
-fn main() { } //~ ERROR compilation successful
+fn main() { }
