@@ -10,14 +10,14 @@
 
 // aux-build:fat_drop.rs
 
-#![feature(core_intrinsics)]
+#![feature(drop_in_place)]
 
 extern crate fat_drop;
 
 fn main() {
     unsafe {
         let s: &mut fat_drop::S = std::mem::uninitialized();
-        std::intrinsics::drop_in_place(s);
+        std::ptr::drop_in_place(s);
         assert!(fat_drop::DROPPED);
     }
 }
