@@ -10,11 +10,6 @@
 
 // Checks that higher-ranked extern fn pointers implement the full range of Fn traits.
 
-
-#![feature(unboxed_closures, core)]
-
-use std::ops::{Fn,FnMut,FnOnce};
-
 fn square(x: &isize) -> isize { (*x) * (*x) }
 
 fn call_it<F:Fn(&isize)->isize>(f: &F, x: isize) -> isize {
@@ -22,7 +17,7 @@ fn call_it<F:Fn(&isize)->isize>(f: &F, x: isize) -> isize {
 }
 
 fn call_it_boxed(f: &Fn(&isize) -> isize, x: isize) -> isize {
-    f.call((&x,))
+    f(&x)
 }
 
 fn call_it_mut<F:FnMut(&isize)->isize>(f: &mut F, x: isize) -> isize {
