@@ -12,7 +12,7 @@
 // any Fn trait to be used where Fn is implemented.
 
 
-#![feature(unboxed_closures, core)]
+#![feature(unboxed_closures, fn_traits)]
 
 use std::ops::{Fn,FnMut,FnOnce};
 
