@@ -8,7 +8,7 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-#![feature(lang_items, start, no_std, core_slice_ext, libc, collections)]
+#![feature(lang_items, start, libc, collections)]
 #![no_std]
 
 extern crate std as other;
@@ -18,8 +18,6 @@ extern crate libc;
 #[macro_use]
 extern crate collections;
 
-use core::option::Option::Some;
-use core::slice::SliceExt;
 use collections::vec::Vec;
 
 // Issue #16806
