@@ -14,7 +14,7 @@
 use std::collections::BinaryHeap;
 
 fn make_pq() -> BinaryHeap<isize> {
-    BinaryHeap::from_vec(vec![1,2,3])
+    BinaryHeap::from(vec![1,2,3])
 }
 
 pub fn main() {
