@@ -9,9 +9,10 @@
 // except according to those terms.
 
 #![feature(staged_api)]
-#![staged_api]
 #![doc(issue_tracker_base_url = "http://issue_url/")]
 
+#![unstable(feature="test", issue="27759")]
+
 // @has issue_27759/unstable/index.html
 // @has - '<code>test</code>'
 // @has - '<a href="http://issue_url/27759">#27759</a>'
